/*
 *   ORXONOX - the hottest 3D action shooter ever to exist
 *                    > www.orxonox.net <
 *
 *
 *   License notice:
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *   Author:
 *      Fabian 'x3n' Landau
 *   Co-authors:
 *      ...
 *
 */

#include "HUDHealthBar.h"

#include "util/Convert.h"
#include "core/CoreIncludes.h"
#include "core/XMLPort.h"
#include "worldentities/pawns/Pawn.h"
#include "overlays/OverlayGroup.h"

namespace orxonox
{
    RegisterClass(HUDHealthBar);

    HUDHealthBar::HUDHealthBar(Context* context) : HUDBar(context)
    {
        RegisterObject(HUDHealthBar);

        this->owner_ = 0;
        this->bUseBarColour_ = false;
        this->textOffset_ = Vector2(0.0f, 0.0f);
        this->textScale_ = 1.0f;

        this->setIconPosition(Vector2(0.05f,0.5f));
        this->setIconDimensions(Vector2(0.1f,0.5f));

        this->textoverlay_ = new OverlayText(this->getContext());

        assert(this->textoverlay_.get());

        this->textoverlay_->setCaption("");
        this->textoverlay_->setAspectCorrection(false);

        positionText();
    }

    HUDHealthBar::~HUDHealthBar()
    {
        if (this->isInitialized())
        {
            this->textoverlay_->destroy();
            this->textoverlay_ = NULL;
        }
    }

    void HUDHealthBar::XMLPort(Element& xmlelement, XMLPort::Mode mode)
    {
        SUPER(HUDHealthBar, XMLPort, xmlelement, mode);

        XMLPortParam(HUDHealthBar, "showtext",          setTextVisible,          getTextVisible,          xmlelement, mode).defaultValues(true);
        XMLPortParam(HUDHealthBar, "textfont",          setTextFont,             getTextFont,             xmlelement, mode).defaultValues("Monofur");
        XMLPortParam(HUDHealthBar, "textusebarcolour",  setTextUseBarColour,     getTextUseBarColour,     xmlelement, mode).defaultValues(false);
        XMLPortParam(HUDHealthBar, "textcolour",        setTextColour,           getTextColour,           xmlelement, mode).defaultValues(ColourValue(1.0, 1.0, 1.0, 1.0));
        XMLPortParam(HUDHealthBar, "textalign",         setTextAlignmentString,  getTextAlignmentString,  xmlelement, mode).defaultValues("left");
        XMLPortParam(HUDHealthBar, "textoffset",        setTextOffset,           getTextOffset,           xmlelement, mode).defaultValues(Vector2::ZERO);
        XMLPortParam(HUDHealthBar, "textscale",         setTextScale,            getTextScale,            xmlelement, mode).defaultValues(1.0f);
        XMLPortParam(HUDHealthBar, "textpickpoint",     setTextPickPoint,        getTextPickPoint,        xmlelement, mode).defaultValues(Vector2::ZERO);
        XMLPortParam(HUDHealthBar, "textrotation",      setTextRotation,         getTextRotation,         xmlelement, mode).defaultValues(0.0f);
        XMLPortParam(HUDHealthBar, "textspacewidth",    setTextSpaceWidth,       getTextSpaceWidth,       xmlelement, mode).defaultValues(true);
    }

    void HUDHealthBar::tick(float dt)
    {
        SUPER(HUDHealthBar, tick, dt);

        if (this->owner_)
        {
            this->setValue(this->owner_->getHealth() / this->owner_->getInitialHealth());
            this->textoverlay_->setCaption(multi_cast<std::string>(static_cast<int>(this->owner_->getHealth())));
        }
        else
        {
            this->setValue(0);
            this->textoverlay_->setCaption("0");
        }

        if (this->bUseBarColour_)
            this->textoverlay_->setColour(this->getCurrentBarColour());
    }

    void HUDHealthBar::changedOwner()
    {
        SUPER(HUDHealthBar, changedOwner);

        this->owner_ = orxonox_cast<Pawn*>(this->getOwner());
    }

    void HUDHealthBar::changedOverlayGroup()
    {
        SUPER(HUDHealthBar, changedOverlayGroup);

        this->getOverlayGroup()->addElement(this->textoverlay_.get());
    }

    void HUDHealthBar::changedVisibility()
    {
        SUPER(HUDHealthBar, changedVisibility);

        this->textoverlay_->setVisible(this->isVisible());
    }

    void HUDHealthBar::changedName()
    {
        SUPER(HUDHealthBar, changedName);

        this->textoverlay_->setName(this->getName() + "text");
    }

    void HUDHealthBar::setTextColour(const ColourValue& colour)
    {
        this->textColour_ = colour;
        if (!this->bUseBarColour_)
            this->textoverlay_->setColour(colour);
    }

    void HUDHealthBar::setTextUseBarColour(bool bUseBarColour)
    {
        this->bUseBarColour_ = bUseBarColour;
        if (!bUseBarColour)
            this->textoverlay_->setColour(this->textColour_);
    }

    void HUDHealthBar::positionText()
    {
        this->textoverlay_->setPosition(this->getPosition() + this->textOffset_*this->getActualSize());
        this->textoverlay_->setTextSize(this->getActualSize().y*this->textScale_);
    }

    void HUDHealthBar::positionChanged()
    {
        HUDBar::positionChanged();
        positionText();
    }

    void HUDHealthBar::sizeChanged()
    {
        HUDBar::sizeChanged();
        positionText();
    }    
}
